#include "includes.h"

/*
** This file contains the definitions about which counters have to be 
** read from /proc
**                                                      
** New counters can be added to the tables in this file as follows:
**       Define if the counter-contents is static (e.g. a kernel-param)
**       and only has to be read once, or if the counter dynamically  
**       changes (to be read per sample). In case of a static counter
**       add its specification to the 'oncedef' table and for dynamic
**       counter, add its specification to the 'sampdef' table.      
**                                                                
** ---------------------------------------------------------------------
** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
** E-mail:      gerlof@ATComputing.nl
** Date:        Januar  1995
** LINUX-port:  Februar 1999
**
** $Log: fetchdef.c,v $
** Revision 1.10  1999/10/18 12:58:53  gerlof
** Separate counters for FTP-input and -output.
**
** Revision 1.9  1999/09/16 09:27:07  gerlof
** Minor code-restyling.
**
** Revision 1.8  1999/09/01 07:12:09  gerlof
** Ported to Alpha.
**
** Revision 1.7  1999/08/31 09:34:29  gerlof
** Implement support for WEB-servers.
** Porting to version 2.0.
**
** Revision 1.6  1999/08/27 14:05:32  gerlof
** Added NFS-statistics.
**
** Revision 1.5  1999/08/26 07:47:19  gerlof
** Code-cleanup, removed all counters obtained from /dev/kmem (obtained via
** /proc now) and gathered new counters (a.o. socket-stats).
**
** Revision 1.4  1999/05/18 08:31:19  gerlof
** Back-port from version 2.2 to 2.0.
**
** Revision 1.3  1999/05/11 09:07:33  gerlof
** Add support for disk-partition statistics and kernel-tables.
**
** Revision 1.2  1999/05/10 10:13:19  gerlof
** Gather statistics about kernel-table occupation (parinfo).
**
** Revision 1.1  1999/05/05 11:38:33  gerlof
** Initial revision
**
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License as published by the
** Free Software Foundation; either version 2, or (at your option) any
** later version.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU General Public License for more details.
*/

static char rcsid[] = "$Id: fetchdef.c,v 1.10 1999/10/18 12:58:53 gerlof Exp $";

#define	EQ	0

/*************************************************************/
/* specific routines which are defined above                 */
/*************************************************************/
#define	MAXCNT	64

/*
** Gather general system statistics from /proc and store in binary form.
*/
void
genget(struct fetchdef *fdef, struct osrel *osrel, struct genstat *gs)
{
	register int	nr, i;
	FILE 		*fp;
	char		linebuf[1024], nam[64];
	unsigned long	cnts[MAXCNT];

	memset(gs, 0, sizeof(struct genstat));

	/*
	** gather various general statistics from the file /proc/stat and
	** store them in binary form in a proprietary structure.
	*/
	if ( (fp = fopen("/proc/stat", "r")) == NULL)
		return;

	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
	{
		nr = sscanf(linebuf,
			"%s  %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
				nam,
				&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
				&cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
				&cnts[8],  &cnts[9],  &cnts[10], &cnts[11],
				&cnts[12], &cnts[13], &cnts[14], &cnts[15],
				&cnts[16], &cnts[17], &cnts[18], &cnts[19],
				&cnts[20], &cnts[21], &cnts[22], &cnts[23],
				&cnts[24], &cnts[25], &cnts[26], &cnts[27],
				&cnts[28], &cnts[29], &cnts[30], &cnts[31],
				&cnts[32], &cnts[33], &cnts[34], &cnts[35],
				&cnts[36], &cnts[37], &cnts[38], &cnts[39],
				&cnts[40], &cnts[41], &cnts[42], &cnts[43],
				&cnts[44], &cnts[45], &cnts[46], &cnts[47],
				&cnts[48], &cnts[49], &cnts[50], &cnts[51],
				&cnts[52], &cnts[53], &cnts[54], &cnts[55],
				&cnts[56], &cnts[57], &cnts[58], &cnts[59],
				&cnts[60], &cnts[61], &cnts[62], &cnts[63]);

		if (nr < 2)		/* headerline ? --> skip */
			continue;

		if ( strcmp("cpu", nam) == EQ)
		{
			gs->cpu_user	= cnts[0];
			gs->cpu_nice	= cnts[1];
			gs->cpu_system	= cnts[2];
			gs->cpu_idle	= cnts[3];

			continue;
		}

		if ( nam[0] == 'c' && nam[1] == 'p' && nam[2] == 'u' )
		{
			int cpunr = atoi( &nam[3] );

			if (cpunr < MAXCPU && cpunr >= 0)
			{
				gs->per_cpu_user[cpunr]		= cnts[0];
				gs->per_cpu_nice[cpunr]		= cnts[1];
				gs->per_cpu_system[cpunr]	= cnts[2];
				gs->per_cpu_idle[cpunr]		= cnts[3];
			}

			continue;
		}

		if ( strcmp("ctxt", nam) == EQ)
		{
			gs->context_swtch	= cnts[0];

			continue;
		}

		if ( strcmp("intr", nam) == EQ)
		{
			int max = (nr-2) < MAXIRQ ? (nr-2) : MAXIRQ;

			/* first counter is the sum of all counters */
			for (i=1; i < max; i++)
				gs->irqs[i-1] = cnts[i];

			continue;
		}

		if ( strcmp("page", nam) == EQ)
		{
			gs->pgpgin	= cnts[0];
			gs->pgpgout	= cnts[1];

			continue;
		}

		if ( strcmp("swap", nam) == EQ)
		{
			gs->pswpin	= cnts[0];
			gs->pswpout	= cnts[1];

			continue;
		}

		if ( strcmp("disk", nam) == EQ)
		{
			int max = (nr-1) < MAXDISK ? (nr-1) : MAXDISK;

			for (i=0; i < max; i++)
				gs->dk_drive[i]		= cnts[i];

			continue;
		}

		if ( strcmp("disk_rio", nam) == EQ)
		{
			int max = (nr-1) < MAXDISK ? (nr-1) : MAXDISK;

			for (i=0; i < max; i++)
				gs->dk_drive_rio[i]	= cnts[i];

			continue;
		}

		if ( strcmp("disk_wio", nam) == EQ)
		{
			int max = (nr-1) < MAXDISK ? (nr-1) : MAXDISK;

			for (i=0; i < max; i++)
				gs->dk_drive_wio[i]	= cnts[i];

			continue;
		}

		if ( strcmp("disk_rblk", nam) == EQ)
		{
			int max = (nr-1) < MAXDISK ? (nr-1) : MAXDISK;

			for (i=0; i < max; i++)
				gs->dk_drive_rblk[i]	= cnts[i];

			continue;
		}

		if ( strcmp("disk_wblk", nam) == EQ)
		{
			int max = (nr-1) < MAXDISK ? (nr-1) : MAXDISK;

			for (i=0; i < max; i++)
				gs->dk_drive_wblk[i]	= cnts[i];

			continue;
		}
	}

	fclose(fp);

	/*
	** gather counters of per-cpu interrupt-statistics
	*/
	if ( (fp = fopen("/proc/interrupts", "r")) == NULL)
		return;

	/* skip header-line */
	if ( fgets(linebuf, sizeof(linebuf), fp) == NULL)
		return;

	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
	{
		int irq, max;

		nr = sscanf(linebuf,
			"%d: %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
				&irq,
				&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
				&cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
				&cnts[8],  &cnts[9],  &cnts[10], &cnts[11],
				&cnts[12], &cnts[13], &cnts[14], &cnts[15],
				&cnts[16], &cnts[17], &cnts[18], &cnts[19],
				&cnts[20], &cnts[21], &cnts[22], &cnts[23],
				&cnts[24], &cnts[25], &cnts[26], &cnts[27],
				&cnts[28], &cnts[29], &cnts[30], &cnts[31],
				&cnts[32], &cnts[33], &cnts[34], &cnts[35],
				&cnts[36], &cnts[37], &cnts[38], &cnts[39],
				&cnts[40], &cnts[41], &cnts[42], &cnts[43],
				&cnts[44], &cnts[45], &cnts[46], &cnts[47],
				&cnts[48], &cnts[49], &cnts[50], &cnts[51],
				&cnts[52], &cnts[53], &cnts[54], &cnts[55],
				&cnts[56], &cnts[57], &cnts[58], &cnts[59],
				&cnts[60], &cnts[61], &cnts[62], &cnts[63]);

			if (nr < 2)
				continue;

			max = (nr-1) < MAXCPU ? (nr-1) : MAXCPU;

			for (i=0; i < max; i++)
				gs->per_irqs[i][irq] = cnts[i];
	}
	fclose(fp);
}

/*
** Gather memory-related statistics from /proc and store in binary form.
*/
void
memget(struct fetchdef *fdef, struct osrel *osrel, struct memstat *mi)
{
	register int	nr, nrfills=2;
	FILE 		*fp;
	char		linebuf[1024];
	char		itsname[32];
	unsigned long	cnts[10];

	memset(mi, 0, sizeof(struct memstat));

	if ( (fp = fopen("/proc/meminfo", "r")) == NULL)
		return;

	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
	{
		nr = sscanf(linebuf,
			"%s %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
			itsname,
			&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],  &cnts[4],
			&cnts[5],  &cnts[6],  &cnts[7],  &cnts[8],  &cnts[9]);

		if (nr < 2)		/* headerline ? --> skip */
			continue;

		if ( strcmp("Mem:", itsname) == EQ)
		{
			memcpy(mi->memory, cnts, sizeof(mi->memory));

			if (--nrfills > 0)
				continue;
			else
				break;
		}

		if ( strcmp("Swap:", itsname) == EQ)
		{
			memcpy(mi->swap, cnts, sizeof(mi->swap));

			if (--nrfills > 0)
				continue;
			else
				break;
		}
	}

	fclose(fp);
}


/*
** Gather network-interface statistics from /proc and store in binary form.
*/
void
ifinit(struct fetchdef *fdef, struct osrel *osrel)
{
	int		numnet=0;
	struct ifstat 	ifs[MAXNET];

	numnet = getifstats(osrel, ifs, MAXNET);

	/*
	** current number of instances defined;
	** however, this number might be changed dynamically, so
	** reserve some spare entries (round up to unit of 8);
	** unused entries will be filled with binary zeroes during
	** each sample
	*/
	fdef->cdef.kinst = ((numnet>>3) + 1) << 3;

	if (fdef->cdef.kinst > MAXNET)
		fdef->cdef.kinst = MAXNET;
}

void
ifget(struct fetchdef *fdef, struct osrel *osrel, struct ifstat *ifsbuf)
{
	memset(ifsbuf, 0, sizeof(struct ifstat) * fdef->cdef.kinst);

	(void) getifstats(osrel, ifsbuf, fdef->cdef.kinst);
}

int
getifstats(struct osrel *osrel, struct ifstat *ifsp, int maxif)
{
	register int	i=0, nr;
	int		notused;
	FILE 		*fp;
	char		linebuf[1024], *cp;

	if ( (fp = fopen("/proc/net/dev", "r")) == NULL)
		return(0);

	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
	{
		if (i >= maxif)
			break;


		if ( (cp = strchr(linebuf, ':')) != NULL)
			*cp = ' ';	/* substitute ':' by space */

		if (osrel->rel == 2 && osrel->vers < 2)
		{
			memset(ifsp, 0, sizeof(struct ifstat));

	                nr = sscanf(linebuf,
				    "%s  %lu %lu %lu %lu %lu %lu %lu "
				    "%lu %lu %lu %lu\n",
                                          ifsp->name,
                                        &(ifsp->rpack),
                                        &(ifsp->rerrs),
                                        &(ifsp->rdrop),
                                        &(ifsp->rfifo),
                                        &(ifsp->rframe),
                                        &(ifsp->spack),
                                        &(ifsp->serrs),
                                        &(ifsp->sdrop),
                                        &(ifsp->sfifo),
                                        &(ifsp->scollis),
                                        &(ifsp->scarrier));

			if (nr < 12)	/* skip header & lines without stats */
			{
				ifsp->name[0] = 0;
				continue;
			}
		}
		else
		{
			nr = sscanf(linebuf,
			 	    "%s %lu %lu %lu %lu %lu %lu %lu %lu %lu "
				    "%lu %lu %lu %lu %lu %lu %lu\n",
					  ifsp->name,
					&(ifsp->rbyte),
					&(ifsp->rpack),
					&(ifsp->rerrs),
					&(ifsp->rdrop),
					&(ifsp->rfifo),
					&(ifsp->rframe),
					&(ifsp->rcompr),
					&(ifsp->rmultic),
					&(ifsp->sbyte),
					&(ifsp->spack),
					&(ifsp->serrs),
					&(ifsp->sdrop),
					&(ifsp->sfifo),
					&(ifsp->scollis),
					&(ifsp->scarrier),
					&(ifsp->scompr));

			if (nr < 16)	/* skip header & lines without stats */
			{
				ifsp->name[0] = 0;
				continue;
			}
		}

		i++;
		ifsp++;
	}

	fclose(fp);

	return i;
}

/*
** Gather ip-icmp-tcp-udp statistics from /proc and store in binary form
** in the snmp mib-structure where the counters are derived from.
*/
void
netget(struct fetchdef *fdef, struct osrel *osrel, struct netstat *ns)
{
	register int	nr;
	FILE 		*fp;
	char		linebuf[1024];
	char		itsname[32];
	unsigned long	cnts[40];

	memset(ns, 0, sizeof(struct netstat));

	if ( (fp = fopen("/proc/net/snmp", "r")) == NULL)
		return;

	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
	{
		nr = sscanf(linebuf,
			"%s  %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
			"%lu %lu\n",
				itsname,
				&cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
				&cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
				&cnts[8],  &cnts[9], &cnts[10],  &cnts[11],
				&cnts[12], &cnts[13], &cnts[14], &cnts[15],
				&cnts[16], &cnts[17], &cnts[18], &cnts[19],
				&cnts[20], &cnts[21], &cnts[22], &cnts[23],
				&cnts[24], &cnts[25], &cnts[26], &cnts[27],
				&cnts[28], &cnts[29], &cnts[30], &cnts[31],
				&cnts[32], &cnts[33], &cnts[34], &cnts[35],
				&cnts[36], &cnts[37], &cnts[38], &cnts[39]);

		if (nr < 2)		/* headerline ? --> skip */
			continue;

		if ( strcmp("Ip:", itsname) == EQ)
		{
			memcpy( &ns->ip, cnts, sizeof(struct ip_mib) );
			continue;
		}

		if ( strcmp("Icmp:", itsname) == EQ)
		{
			memcpy( &ns->icmp, cnts, sizeof(struct icmp_mib) );
			continue;
		}

		if ( strcmp("Tcp:", itsname) == EQ)
		{
			memcpy( &ns->tcp, cnts, sizeof(struct tcp_mib) );
			continue;
		}

		if ( strcmp("Udp:", itsname) == EQ)
		{
			memcpy( &ns->udp, cnts, sizeof(struct udp_mib) );
			continue;
		}
	}

	fclose(fp);
}

/*
** Gather nfs (& rpc) statistics from /proc and store in binary form
** in the proprietary nfsstat structure.
*/
void
nfsget(struct fetchdef *fdef, struct osrel *osrel, struct nfsstat *ns)
{
	register int	nr;
	FILE 		*fp;
	char		linebuf[1024];
	char		itsname[32];
	unsigned long	cnts[32];
	void		nfsfill(char *, unsigned long [],
					struct nfsstat *, int);


	memset(ns, 0, sizeof(struct nfsstat));

	/*
	** Get client statistics
	*/
	if ( (fp = fopen("/proc/net/rpc/nfs", "r")) != NULL)
	{
		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
		{
			nr = sscanf(linebuf,
				"%s  %lu %lu %lu %lu %lu %lu %lu %lu %lu "
				"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
				"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
				"%lu %lu %lu\n",
				   itsname,
				   &cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
				   &cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
				   &cnts[8],  &cnts[9], &cnts[10],  &cnts[11],
				   &cnts[12], &cnts[13], &cnts[14], &cnts[15],
				   &cnts[16], &cnts[17], &cnts[18], &cnts[19],
				   &cnts[20], &cnts[21], &cnts[22], &cnts[23],
				   &cnts[24], &cnts[25], &cnts[26], &cnts[27],
				   &cnts[28], &cnts[29], &cnts[30], &cnts[31]);

			if (nr < 2)
				continue;

			nfsfill(itsname, cnts, ns, 1);
		}
	}

	/*
	** Get server statistics
	*/
	if ( (fp = fopen("/proc/net/rpc/nfsd", "r")) != NULL)
	{
		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
		{
			nr = sscanf(linebuf,
				"%s  %lu %lu %lu %lu %lu %lu %lu %lu %lu "
				"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
				"%lu %lu %lu %lu %lu %lu %lu %lu %lu %lu "
				"%lu %lu %lu\n",
				   itsname,
				   &cnts[0],  &cnts[1],  &cnts[2],  &cnts[3],
				   &cnts[4],  &cnts[5],  &cnts[6],  &cnts[7],
				   &cnts[8],  &cnts[9], &cnts[10],  &cnts[11],
				   &cnts[12], &cnts[13], &cnts[14], &cnts[15],
				   &cnts[16], &cnts[17], &cnts[18], &cnts[19],
				   &cnts[20], &cnts[21], &cnts[22], &cnts[23],
				   &cnts[24], &cnts[25], &cnts[26], &cnts[27],
				   &cnts[28], &cnts[29], &cnts[30], &cnts[31]);

			if (nr < 2)
				continue;

			nfsfill(itsname, cnts, ns, 0);
		}
	}
}

void
nfsfill(char *nam, unsigned long cnts[], struct nfsstat *ns, int isclient)
{
	if ( strcmp("net", nam) == EQ)
	{
		if (isclient)
		{
			ns->cl_netcnt		= cnts[0];
			ns->cl_netudpcnt	= cnts[1];
			ns->cl_nettcpcnt	= cnts[2];
			ns->cl_nettcpconn	= cnts[3];
		}
		else
		{
			ns->sv_netcnt		= cnts[0];
			ns->sv_netudpcnt	= cnts[1];
			ns->sv_nettcpcnt	= cnts[2];
			ns->sv_nettcpconn	= cnts[3];
		}

		return;
	}

	if ( strcmp("rpc", nam) == EQ)
	{
		if (isclient)
		{
			ns->cl_rpccnt		= cnts[0];
			ns->cl_rpcretrans	= cnts[1];
			ns->cl_rpcauthrefresh	= cnts[2];
		}
		else
		{
			ns->sv_rpccnt		= cnts[0];
			ns->sv_rpcbadtot	= cnts[1];
			ns->sv_rpcbadfmt	= cnts[2];
			ns->sv_rpcbadauth	= cnts[3];
			ns->sv_rpcbadclnt	= cnts[4];
		}

		return;
	}

	if ( nam[0] == 'p' && nam[1] == 'r' &&
	     nam[2] == 'o' && nam[3] == 'c'   )
	{
		int version = atoi(nam+4);

		switch (version)	{
		   case 2:
			if (isclient)
				memcpy(&ns->cl_proc2, &cnts[1],
						sizeof(ns->cl_proc2) );
			else
				memcpy(&ns->sv_proc2, &cnts[1],
						sizeof(ns->sv_proc2) );
			break;

		   case 3:
			if (isclient)
				memcpy(&ns->cl_proc3, &cnts[1],
						sizeof(ns->cl_proc3) );
			else
				memcpy(&ns->sv_proc3, &cnts[1],
						sizeof(ns->sv_proc3) );
			break;
		}

		return;
	}
}

/*
** Gather statistics about the utilization of parameters from /proc and
** store in binary form.
*/
void
tabget(struct fetchdef *fdef, struct osrel *osrel, struct tabstat *ps)
{
	register int	nr, baselen;
	FILE 		*fp;
	char		linebuf[1024], firstword[80], basepath[132];
	long		cnt;

	memset(ps, 0, sizeof(struct tabstat));

	/*
	** obtain info about number of processes started
	*/
	if ( (fp = fopen("/proc/stat", "r")) != NULL)
	{
		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
		{
			sscanf(linebuf, "%s %lu", firstword, &cnt);
	
			if ( strcmp(firstword, "processes") == EQ)
			{
				ps->totforks = cnt;
				break;
			}
		}
		fclose(fp);
	}

	/*
	** obtain info about number of record-locks
	*/
	cnt = 0;

	if ( (fp = fopen("/proc/locks", "r")) != NULL)
	{
		/* just count number of lines */
		while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
			cnt++;

		fclose(fp);
	}

	ps->curlocks = cnt;

	/*
	** obtain info about occupation of disk-quota
	*/
	if ( (fp = fopen("/proc/sys/fs/dquot-max", "r")) != NULL)
	{
		fscanf(fp, "%lu", &(ps->maxdquot));
		fclose(fp);
	}

	if ( (fp = fopen("/proc/sys/fs/dquot-nr", "r")) != NULL)
	{
		fscanf(fp, "%lu", &(ps->curdquot));
		fclose(fp);
	}

	/*
	** obtain info about occupation of superblocks
	*/
	if ( (fp = fopen("/proc/sys/fs/super-max", "r")) != NULL)
	{
		fscanf(fp, "%lu", &(ps->maxsuper));
		fclose(fp);
	}

	if ( (fp = fopen("/proc/sys/fs/super-nr", "r")) != NULL)
	{
		fscanf(fp, "%lu", &(ps->cursuper));
		fclose(fp);
	}

	/*
	** Next file-names under /proc are dependent on the release/version
	** of this system; they might be stored under /proc/sys/kernel or
	** under /proc/sys/fs. General preparation:
	*/
	if (osrel->rel == 2 && osrel->vers < 2)
		strcpy(basepath, "/proc/sys/kernel/");
	else
		strcpy(basepath, "/proc/sys/fs/");

	baselen = strlen(basepath);

	/*
	** obtain info about occupation of open files
	*/
	strcpy(&basepath[baselen], "file-max");

	if ( (fp = fopen(basepath, "r")) != NULL)
	{
		fscanf(fp, "%lu", &(ps->maxfiles));
		fclose(fp);
	}

	strcpy(&basepath[baselen], "file-nr");

	if ( (fp = fopen(basepath, "r")) != NULL)
	{
		fscanf(fp, "%lu", &(ps->curfiles));
		fclose(fp);
	}

	/*
	** obtain info about occupation of inodes
	*/
	strcpy(&basepath[baselen], "inode-max");

	if ( (fp = fopen(basepath, "r")) != NULL)
	{
		fscanf(fp, "%lu", &(ps->maxinode));
		fclose(fp);
	}

	strcpy(&basepath[baselen], "inode-nr");

	if ( (fp = fopen(basepath, "r")) != NULL)
	{
		fscanf(fp, "%lu", &(ps->curinode));
		fclose(fp);
	}
}

/*
** Gather statistics about the utilization of disk-partitions from /proc and
** store in binary form.
*/
void
dkinit(struct fetchdef *fdef, struct osrel *osrel)
{
	int		numpart=0, nr=0;
	FILE		*fp;
	struct dkstat	dkstat;
	char		buf[256];

	/*
	** define number of partitions and whether or not partition-stats
	** are supported on this system;
	*/
	if ( (fp = fopen("/proc/partitions", "r")) == NULL)
	{
		fdef->cdef.kinst = 0;
		return;
	}

	while ( fgets(buf, sizeof(buf), fp) )
	{
		nr = sscanf(buf, "%d  %d  %lu %63s %lu %lu %lu %lu %lu "
				 "%lu %lu %lu %lu  %lu %lu",
                       		&dkstat.major,
				&dkstat.minor,
                       		&dkstat.totsize,
                       		 dkstat.name,

                       		&dkstat.readblocks,
                       		&dkstat.readmerges,
                       		&dkstat.readsectors,
                       		&dkstat.readticks,

                       		&dkstat.writblocks,
                       		&dkstat.writmerges,
                       		&dkstat.writsectors,
                       		&dkstat.writticks,

                       		&dkstat.reqpending,
                       		&dkstat.rdwrticks,
                       		&dkstat.avq);

		if (nr >= 15)
			numpart++;
	}
	fclose(fp);

	fdef->cdef.kinst = numpart;
}

void
dkget(struct fetchdef *fdef, struct osrel *osrel, struct dkstat *ds)
{
	FILE	*fp;
	int	numpart=0, nr=0;
	char	buf[256];

	if (fdef->cdef.kinst == 0)
		return;

	memset(ds, 0, sizeof(struct dkstat) * fdef->cdef.kinst);

	if ( (fp = fopen("/proc/partitions", "r")) == NULL)
		return;

	while ( fgets(buf, sizeof(buf), fp) && numpart < fdef->cdef.kinst)
	{
		nr = sscanf(buf, "%d  %d  %lu %63s %lu %lu %lu %lu %lu "
				 "%lu %lu %lu %lu  %lu %lu",
                      		&(ds->major),
				&(ds->minor),
                       		&(ds->totsize),
                       		  ds->name,

                       		&(ds->readblocks),
                       		&(ds->readmerges),
                       		&(ds->readsectors),
                       		&(ds->readticks),

                       		&(ds->writblocks),
                       		&(ds->writmerges),
                       		&(ds->writsectors),
                       		&(ds->writticks),

                       		&(ds->reqpending),
                       		&(ds->rdwrticks),
                       		&(ds->avq));

		if (nr == 15)
		{
			numpart++;
			ds++;
		}
	}
	fclose(fp);
}

/*
** Gather statistics about the utilization of sockets from /proc and
** store in binary form.
*/
void
sockget(struct fetchdef *fdef, struct osrel *osrel, struct sockstat *ss)
{
	FILE 		*fp;
	char		linebuf[1024];
	char		itsname[32], notused[64];
	unsigned long	nowuse, maxuse;

	memset(ss, 0, sizeof(struct sockstat));

	if ( (fp = fopen("/proc/net/sockstat", "r")) == NULL)
		return;

	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
	{
		if ( sscanf(linebuf, "%s  %s %lu %s %lu\n",
			itsname, notused, &nowuse, notused, &maxuse) < 5)
				continue;

		if ( strcmp("TCP:", itsname) == EQ)
		{
			ss->tcpnow = nowuse;
			ss->tcpmax = maxuse;
			continue;
		}

		if ( strcmp("UDP:", itsname) == EQ)
		{
			ss->udpnow = nowuse;
			ss->udpmax = maxuse;
			continue;
		}
	}
	fclose(fp);
}


#ifdef	WEBSUPPORT
/*
** Gather statistics about the number of FTP-transfers
*/
void
ftpget(struct fetchdef *fdef, struct osrel *osrel, struct ftpstat *fs)
{
	FILE 		*fp;
	char		linebuf[1024];

	memset(fs, 0, sizeof(struct ftpstat));

	if ( (fp = fopen("/var/log/atsar/ftpstat", "r")) == NULL)
		return;

	if ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
	{
		fs->valid = 1;
		sscanf(linebuf, "%lu %lu %lu %lu %lu %lu\n",
			&fs->ocumxfer, &fs->ocumseconds, &fs->ocumkbytes,
			&fs->icumxfer, &fs->icumseconds, &fs->icumkbytes);
	}
	fclose(fp);
}

/*
** Gather HTTP-statistics
*/
void
httpinit(struct fetchdef *fdef, struct osrel *osrel)
{
	FILE 		*fp;
	char		linebuf[1024];

	fdef->cdef.kinst = 0;

	if ( (fp = fopen("/var/log/atsar/httpstat", "r")) == NULL)
		return;

	while ( fgets(linebuf, sizeof(linebuf), fp) != NULL)
		fdef->cdef.kinst++;

	fclose(fp);
}

void
httpget(struct fetchdef *fdef, struct osrel *osrel, struct httpstat *hs)
{
	register int	i;
	FILE 		*fp;
	char		linebuf[1024], namebuf[512], *p;
	unsigned long	notused;


	if ( (fp = fopen("/var/log/atsar/httpstat", "r")) == NULL)
		return;

	for (i=0; i < fdef->cdef.kinst; i++, hs++)
	{
		memset(hs, 0, sizeof(struct httpstat));

		if  ( fgets(linebuf, sizeof(linebuf), fp) == NULL)
			break;

		sscanf(linebuf, "%lu %lu %lu %lu %lu %lu %lu %lu %s\n",
			&hs->cumhead, &hs->cumgets, &hs->cumputs,
			&hs->cumpost, &hs->cumdels, &notused,
			&notused, &notused, namebuf);

		if ( (p = strrchr(namebuf, '/')) != NULL)
		{
			*p = 0;
			p++;
		}
		else
			p = namebuf;

		strncpy(hs->logname, p, MAXACCNAME-1);
	}

	fclose(fp);
}
#endif


/*********************************************************************/
/* Counter fetch definition tables.                                  */
/*                                                                   */
/* The layout of these tables is as follows:                         */
/*     Column 1:                                                     */
/*        Internal name of counter-set, to be filled by atsadc and   */
/*        to be printed by atsar.                                    */
/*                                                                   */
/*     Column 2:                                                     */
/*        Size (number of bytes) per instance of this counter or     */
/*        counter-structure.                                         */
/*                                                                   */
/*     Column 3:                                                     */
/*        Number of instances of this counter or counter-structure   */
/*        (e.g. if this is a table of counters).                     */
/*                                                                   */
/*     Column 4:                                                     */
/*        Entry-point (optional) of a function which may initialise  */
/*        this table-entry before its contents is taken into account.*/
/*        If e.g. the number of instances (column 3) can only be     */
/*        determined at runtime, a special function can be written   */
/*        to determine this value.                                   */
/*                                                                   */
/*        This function will be called with two arguments, i.e. a    */
/*        pointer to the relevant fetchdef[] table entry to be       */
/*        filled or modified, and info about the OS-release.         */
/*                                                                   */
/*        The return-value is not relevant.                          */
/*                                                                   */
/*     Column 5:                                                     */
/*        Entry-point (mandatory!) of a function which can be called */
/*        to read the statistical counters during every sample.      */
/*                                                                   */
/*        This function will be called with three arguments, i.e.    */
/*        a pointer to the relevant fetchdef[] table entry to be     */
/*        used for information, info about the OS-release and a      */
/*        pointer to the area in which the counter-data should be    */
/*        stored.                                                    */
/*                                                                   */
/*        The return-value is not relevant.                          */
/*********************************************************************/

	/*******************************************************/
	/* counters which are different on every system, but   */
	/* do not change on-the-fly (only read once).          */
	/*******************************************************/
struct fetchdef oncedef[] =
{

};

int	oncecnt = sizeof(oncedef)/sizeof(struct fetchdef);

	/*******************************************************/
	/* counters which hold the real statistics which       */
	/* constantly increase (read every <t> seconds).       */
	/*******************************************************/
struct fetchdef sampdef[] =
{
	"genstat",	sizeof(struct genstat),		1, 0,        genget,
	"memstat",	sizeof(struct memstat),		1, 0,        memget,
	"ifstat",	sizeof(struct ifstat),		1, ifinit,   ifget,
	"netstat",	sizeof(struct netstat),		1, 0,        netget,
	"nfsstat",	sizeof(struct nfsstat),		1, 0,  	     nfsget,
	"tabstat",	sizeof(struct tabstat),		1, 0,        tabget,
	"partstat",	sizeof(struct dkstat),		1, dkinit,   dkget,
	"sockstat",	sizeof(struct sockstat),	1, 0,  	     sockget,
 
#ifdef	WEBSUPPORT
	"ftpstat",	sizeof(struct ftpstat),		1, 0,  	     ftpget,
	"httpstat",	sizeof(struct httpstat),	1, httpinit, httpget,
#endif
};

int	sampcnt = sizeof(sampdef)/sizeof(struct fetchdef);
